﻿Type=Class
Version=2.02
@EndOfDesignText@
'Class module
Sub Class_Globals
	Public gv As GameView
	Public pad As GamePad
	Private mainTimer As Timer
	Private fps As Float
	fps = 30 'this value is not really important. It is just an approximate used when starting t
	Private lastTime As Long
	Private lblFPS As Label
	Public Asteroids As List
	Private AsteroidsBitmaps(,) As BitmapData
	Private MyShip As Ship
	Public missileBitmap() As Bitmap
	Public missileRect(2) As Rect
	Public missiles As List
	Private sounds As SoundPool
	Public shotSoundId, breakSoundId, crashSoundId As Int
End Sub

Public Sub Initialize (vGV As GameView, vPad As GamePad, vlblFPS As Label)
	GameUtils.init
	gv = vGV
	pad = vPad
	lblFPS = vlblFPS
	mainTimer.Initialize("MainTimer", 15)
	AsteroidsBitmaps = GameUtils.LoadSpritesFromSheet(LoadBitmap(File.DirAssets, "Asteroids.png"), 4, 4)
	Asteroids.Initialize
	missileBitmap = Array As Bitmap(LoadBitmap(File.DirAssets, "missile1.png"), _ 
		LoadBitmap(File.DirAssets, "missile2.png"))
	gv.BitmapsData.Add(CreateBackground)
	Dim bd As BitmapData
	MyShip.Initialize(bd , Me)
	gv.BitmapsData.Add(bd)
	missiles.Initialize
	'sounds are loaded to a SoundPool
	sounds.Initialize(4)
	shotSoundId = sounds.Load(File.DirAssets, "shotgun.mp3")
	breakSoundId = sounds.Load(File.DirAssets, "break.mp3")
	crashSoundId = sounds.Load(File.DirAssets, "crash.mp3")
	
	'add the game pad 
	pad.Initialize(gv)
	Dim length, rightPadding, leftPadding, bottomPadding As Int
	length = 100 * GameUtils.scale 'also the panel's height
	rightPadding = 50 * GameUtils.scale
	leftPadding = 20 * GameUtils.scale
	bottomPadding = 20 * GameUtils.scale
	pad.AddToGameView (gv, leftPadding, 100%y - bottomPadding - length, _
		100%x - rightPadding - leftPadding, length)
	
	StartGame
End Sub

Private Sub StartGame
	'Clear the previous remaining asetroids.
	For i = 0 To Asteroids.Size - 1
		Dim ast As Asteroid
		ast = Asteroids.Get(i)
		ast.RemoveFromGameView 'This method will remove the asteroid from the GameView
	Next
	Asteroids.Clear
	'Create 4 new asteroids
	For i = 1 To 4
		AddAsteroid (Rnd(1%x, 99%x), Rnd(1%y, 99%y), _
			Rnd(50 * GameUtils.scale, 150 * GameUtils.scale))
	Next
	
	MyShip.ResetShip
End Sub

Public Sub AddAsteroid (x As Int, y As Int, size As Int)
	Dim animator As SpriteAnimator
	Dim rects(4) As Rect
	Dim bitmaps(4) As Bitmap
	Dim row As Int
	row = Rnd(0, 4)
	'The asteroids sprite sheet includes 4 rows of sprites.
	'One of the rows is randomly selected.
	For i = 0 To 3
		rects(i) = AsteroidsBitmaps(i, row).SrcRect
		bitmaps(i) = AsteroidsBitmaps(i, row).Bitmap
	Next
	
	animator.Initialize
	Dim bd As BitmapData
	animator.SetFrames(bd, rects, bitmaps) 'sets the frames that will be animated
	animator.AnimationInterval = 10
	Dim ast As Asteroid
	ast.Initialize(bd, animator, Me, x, y, size)
	Asteroids.Add(ast) 'add the asteroid to the asteroids list.
	gv.BitmapsData.Add(bd) 'add the asteroid to the GameView (so it will be redrawn every time).
End Sub

Public Sub StartTimer
	mainTimer.Enabled = True
End Sub

Public Sub StopTimer
	mainTimer.Enabled = False
End Sub

Private Sub mainTimer_Tick
	'calculate the frames per second
	fps = (1000 / Max(10, (DateTime.Now - lastTime)) + 20 * fps) / 21
	lblFPS.Text = NumberFormat(fps, 0, 0)
	lastTime = DateTime.Now
	
	Dim ast As Asteroid
	Dim i As Int
	i = 0
	'if there are no more asteroids then the game is completed (the player has destroyed all of them).
	If Asteroids.Size = 0 Then
		Msgbox("Well done!", "")
		StartGame
		Return
	End If
	'go over all the asteroids and call their tick method.
	Do While i < Asteroids.Size
		ast = Asteroids.Get(i)
		'if the asteroid delete flag is on then we remove it from the list.
		If ast.delete Then
			Asteroids.RemoveAt(i)
		Else
			ast.Tick
			i = i + 1
		End If
	Loop
	'similar implementation for the missiles.
	Dim mis As Missile
	i = 0
	Do While i < missiles.Size
		mis = missiles.Get(i)
		If mis.delete Then
			missiles.RemoveAt(i) 'remove missile
		Else
			mis.Tick
			i = i + 1
		End If
	Loop
	'call the ship tick method.
	MyShip.Tick
	'check whether the ship is in the crash state.
	If MyShip.state = MyShip.STATE_CRASHED Then
		Msgbox("Try again...", "")
		StartGame
	End If
	'mark the GameView as "dirty". This will cause it to redraw itself.
	gv.Invalidate
End Sub

Public Sub PlaySound(sound As Int, volume As Float)
	sounds.Play(sound, volume, volume, 0, 0, 1)
End Sub

'Create the stars background
Private Sub CreateBackground As BitmapData
	'The background is created manually with a bitmap and canvas.
	Dim bmp As Bitmap
	bmp.InitializeMutable(100%x, 100%y)
	Dim cvs As Canvas
	cvs.Initialize2(bmp)
	cvs.DrawColor(Colors.Black)
	For i = 1 To 100
		cvs.DrawCircle(Rnd(0, 100%x), Rnd(0, 100%y), 2dip, Colors.White, True, 0)
	Next
	Dim bd As BitmapData
	bd.Bitmap = bmp
	bd.DestRect.Initialize(0, 0, 100%x, 100%y)
	Return bd
End Sub



