﻿Type=Class
Version=2.02
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private animator As SpriteAnimator
	Private shipBd As BitmapData
	Private game As GameManager
	Private MAX_SPEED As Int
	MAX_SPEED = 4 * GameUtils.scale
	Private speed As Float
	Private shipBmp, shipForwardBmp, shipBackwardBmp As Bitmap
	Private currentBmp As Bitmap
	Private shipRect As Rect
	Private ticks, lastFire, fireInterval, explodeStart As Int
	fireInterval = 20
	Private gracePeriod As Int : gracePeriod = 180
	Private missileSpeed As Int
	missileSpeed = 5 * GameUtils.scale
	Private collision As CollisionDetector
	Private crashBitmaps(,) As BitmapData
	Public state As Int
	Public STATE_NORMAL, STATE_EXPLODING, STATE_CRASHED As Int
	STATE_NORMAL = 0 : STATE_EXPLODING = 1: STATE_CRASHED = 2
End Sub

Public Sub Initialize (vBD As BitmapData, vGame As GameManager)
	animator.Initialize
	shipBd = vBD
	shipBmp = LoadBitmap(File.DirAssets, "ship.png")
	shipForwardBmp = LoadBitmap(File.DirAssets, "ship_forward.png")
	shipBackwardBmp = LoadBitmap(File.DirAssets, "ship_back.png")
	
	currentBmp = shipBmp
	animator.width = 30 * GameUtils.scale 'set the sprite width and height
	animator.height = 45 * GameUtils.scale
	Dim x, y As Int
	x = 50%x - animator.width / 2 'start in the middle of the screen
	y = 50%y - animator.height / 2
	shipBd.DestRect.Initialize(x, y , x + animator.width, y + animator.height)
	game = vGame
	'initialize the ship's collistion detector. It will detect collisions between the ship and the asteroids.
	'passing a SafeZone value of 10 * GameUtils.scale makes the asteroids a bit less harmful.
	collision.Initialize(shipBd.DestRect, game.Asteroids, 10, 10 * GameUtils.scale)
	'load the bitmaps that will be used when the ship crashes.
	crashBitmaps = GameUtils.LoadSpritesFromSheet(LoadBitmap(File.DirAssets, "crash.png"), 7, 1)
End Sub

Public Sub ResetShip
	ticks = 0
	state = STATE_NORMAL
	lastFire = -100
	animator.AnimationInterval = 1
	'note that the ship is drawn from a single frame.
	'shipRect is never initialized so GameView draws the complete bitmap.
	animator.SetFrames(shipBd, Array As Rect(shipRect), Array As Bitmap(shipBmp))
End Sub

Public Sub Tick
	ticks = ticks + 1
	If state = STATE_NORMAL Then
		'check the left joystick and change the rotation and speed
		Dim dx, dy As Int
		Select game.pad.GetLeftValue
			Case game.pad.LEFT
				shipBd.Rotate = shipBd.Rotate - 2
			Case game.pad.RIGHT
				shipBd.Rotate = shipBd.Rotate + 2
			Case game.pad.UP
				speed = Min(speed + 0.2 * GameUtils.scale, MAX_SPEED)
				animator.SetFrames(shipBd, Array As Rect(shipRect), Array As Bitmap(shipForwardBmp))
				currentBmp = shipForwardBmp
			Case game.pad.DOWN
				speed = Max(speed - 0.2 * GameUtils.scale, -MAX_SPEED)
				animator.SetFrames(shipBd, Array As Rect(shipRect), Array As Bitmap(shipBackwardBmp))
				currentBmp = shipBackwardBmp
			Case game.pad.NONE
				If currentBmp <> shipBmp Then
					'
					animator.SetFrames(shipBd, Array As Rect(shipRect), Array As Bitmap(shipBmp))
					currentBmp = shipBmp
				End If
		End Select
		
		If ticks - lastFire > fireInterval Then
			lastFire = ticks
			'check whether the player is shooting
			Select game.pad.GetRightValue
				Case game.pad.UP
					ShootMissile(shipBd.Rotate)
				Case game.pad.RIGHT
					ShootMissile(shipBd.Rotate + 90)
				Case game.pad.DOWN
					ShootMissile(shipBd.Rotate + 180)
				Case game.pad.LEFT
					ShootMissile(shipBd.Rotate + 270)
			End Select
		End If 
		'set the ship speed
		animator.vx = speed * SinD(shipBd.Rotate)
		animator.vy = -speed * CosD(shipBd.Rotate)
		
		'Calling Collision.Tick actually checks for collisions.
		If collision.Tick <> Null AND ticks > gracePeriod Then
			Crash
		End If
		
	Else If state = STATE_EXPLODING Then
		'STATE_EXPLODING is a two seconds state that shows the ship burning before stopping the game.
		If ticks - explodeStart > 120 Then
			state = STATE_CRASHED
		End If
	End If
	animator.Tick(shipBd) 'move the ship
End Sub

Private Sub Crash
	state = STATE_EXPLODING
	game.PlaySound(game.crashSoundId, 0.3)
	Dim rects(4) As Rect
	Dim bitmaps(4) As Bitmap
	For i = 0 To 3
		rects(i) = crashBitmaps(i, 0).SrcRect
		bitmaps(i) = crashBitmaps(i, 0).Bitmap
	Next
	'Change the ship animation frames.
	animator.SetFrames(shipBd, rects, bitmaps)
	animator.AnimationInterval = 10
	explodeStart = ticks
End Sub

Private Sub ShootMissile (offset As Int)
	game.PlaySound(game.shotSoundId, 0.3)
	'create a new Missile and add it to GameView and GameManager.
	Dim msl As Missile
	Dim missileAnimator As SpriteAnimator
	missileAnimator.Initialize
	Dim mbd As BitmapData
	missileAnimator.SetFrames(mbd, game.missileRect, game.missileBitmap)
	mbd.Rotate = offset
	msl.Initialize(mbd, missileAnimator, game)
	Dim x, y As Int
	x = shipBd.DestRect.CenterX
	y = shipBd.DestRect.CenterY
	mbd.DestRect.Initialize(x, y, x + missileAnimator.width, y + missileAnimator.height)
	missileAnimator.vx = missileSpeed * SinD(offset)
	missileAnimator.vy = -missileSpeed * CosD(offset)
	game.gv.BitmapsData.Add(mbd)
	game.missiles.Add(msl)
End Sub